AVM_SOURCE:=$(call qstrip,$(subst $(_space),\ ,$(FREETZ_DL_KERNEL_SOURCE)))
AVM_KERNEL_VERSION:=$(subst .,\.,$(KERNEL_VERSION))

AVM_UNPACK__INT_.gz:=z
AVM_UNPACK__INT_.bz2:=j

KERNEL_SUBVERSION:=iln6
KERNEL_MAKE_DIR:=$(MAKE_DIR)/linux
KERNEL_BUILD_DIR:=$(KERNEL_DIR)
KERNEL_BUILD_ROOT_DIR:=$(KERNEL_BUILD_DIR)/linux-$(KERNEL_VERSION)

KERNEL_IMAGE:=vmlinux.eva_pad
KERNEL_TARGET_BINARY:=kernel-$(KERNEL_LAYOUT)-$(AVM_VERSION).bin
KERNEL_CONFIG_FILE:=$(KERNEL_MAKE_DIR)/Config.$(KERNEL_LAYOUT).$(AVM_VERSION)

KERNEL_COMMON_MAKE_OPTIONS := -C $(KERNEL_BUILD_ROOT_DIR)
KERNEL_COMMON_MAKE_OPTIONS += CROSS_COMPILE="$(KERNEL_CROSS)"
KERNEL_COMMON_MAKE_OPTIONS += KERNEL_MAKE_PATH="$(KERNEL_MAKE_PATH):$(PATH)"
KERNEL_COMMON_MAKE_OPTIONS += ARCH="$(KERNEL_ARCH)"
# TODO: KERNEL_LAYOUT is referenced just once in kernel's makefiles.
# It causes additional fusiv-sources to be added to the list of sources
# to compile. Compiling these sources however fails, that's the reason
# the following line is commented out.
#KERNEL_COMMON_MAKE_OPTIONS += KERNEL_LAYOUT="$(KERNEL_LAYOUT)"
KERNEL_COMMON_MAKE_OPTIONS += INSTALL_HDR_PATH=$(KERNEL_HEADERS_DEVEL_DIR)
KERNEL_COMMON_MAKE_OPTIONS += INSTALL_MOD_PATH="$(FREETZ_BASE_DIR)/$(KERNEL_DIR)"

$(DL_FW_DIR)/$(AVM_SOURCE): | $(DL_FW_DIR)
	@$(call _ECHO, downloading...)
	$(DL_TOOL) $(DL_FW_DIR) $(FREETZ_DL_KERNEL_SOURCE) $(FREETZ_DL_KERNEL_SITE) $(FREETZ_DL_KERNEL_SOURCE_MD5) $(SILENT)

# Make sure that a perfectly clean build is performed whenever Freetz package
# options have changed. The safest way to achieve this is by starting over
# with the source directory.
$(KERNEL_DIR)/.unpacked: $(DL_FW_DIR)/$(AVM_SOURCE) | gcc-kernel
	$(RM) -r $(KERNEL_DIR)
	mkdir -p $(KERNEL_BUILD_DIR)
	@$(call _ECHO,checking structure... )
	@KERNEL_SOURCE_CONTENT=` \
		tar -t$(AVM_UNPACK__INT_$(suffix $(strip $(FREETZ_DL_KERNEL_SOURCE)))) \
			-f $(DL_FW_DIR)/$(FREETZ_DL_KERNEL_SOURCE)| \
		grep -e '^.*\(GPL-\(release_\|\)kernel\.tar\.gz\|linux-$(AVM_KERNEL_VERSION)/\)$$'|head -n1`; \
	if [ -z "$${KERNEL_SOURCE_CONTENT}" ]; then \
		$(call ERROR,1,KERNEL_SOURCE_CONTENT is empty) \
	else \
		$(call _ECHO, unpacking... ) \
		if [ ! -z $$(echo "$$KERNEL_SOURCE_CONTENT"|grep -e '.*\/GPL-\(release_\|\)kernel\.tar\.gz') ]; then \
			tar	-O $(VERBOSE) \
				-x$(AVM_UNPACK__INT_$(suffix $(strip $(FREETZ_DL_KERNEL_SOURCE)))) \
				-f $(DL_FW_DIR)/$(FREETZ_DL_KERNEL_SOURCE) \
				--wildcards "*/$${KERNEL_SOURCE_CONTENT##*/}" | \
			tar	-C $(KERNEL_BUILD_DIR) $(VERBOSE) \
				-xz \
				--transform="s|^.*\(linux-$(AVM_KERNEL_VERSION)/\)|\1|g" --show-transformed; \
		else \
			tar	-C $(KERNEL_BUILD_DIR) $(VERBOSE) \
				-x$(AVM_UNPACK__INT_$(suffix $(strip $(FREETZ_DL_KERNEL_SOURCE)))) \
				-f $(DL_FW_DIR)/$(FREETZ_DL_KERNEL_SOURCE) \
				--transform="s|^.*\(linux-$(AVM_KERNEL_VERSION)/\)|\1|g" --show-transformed \
				"$$KERNEL_SOURCE_CONTENT"; \
		fi \
	fi
	@if [ ! -d $(KERNEL_BUILD_ROOT_DIR) ]; then \
		$(call ERROR,1,KERNEL_BUILD_ROOT_DIR has wrong structure) \
	fi
	@$(call _ECHO, preparing... )
	@set -e; shopt -s nullglob; for i in $(KERNEL_MAKE_DIR)/patches/$(KERNEL_VERSION)/*.patch; do \
		$(PATCH_TOOL) $(KERNEL_BUILD_DIR) $$i; \
	done
	#Version specific patches
	@set -e; shopt -s nullglob; for i in $(KERNEL_MAKE_DIR)/patches/$(KERNEL_VERSION)/$(AVM_VERSION)/*.patch; do \
		$(PATCH_TOOL) $(KERNEL_BUILD_DIR) $$i; \
	done
	@for i in $(KERNEL_LINKING_FILES); do \
		if [ -e $(KERNEL_BUILD_ROOT_DIR)/$$i -a \
		! -e $(KERNEL_BUILD_ROOT_DIR)/include/linux/$${i##*\/linux_} ]; then \
			$(call MESSAGE, Linking  $(KERNEL_BUILD_ROOT_DIR)/include/linux/$${i##*\/linux_}); \
			ln -sf ../../$$i $(KERNEL_BUILD_ROOT_DIR)/include/linux/$${i##*\/linux_}; \
		fi \
	done
	@for i in avm_net_trace.h avm_net_trace_ioctl.h; do \
		if [ -e $(KERNEL_BUILD_ROOT_DIR)/drivers/char/avm_net_trace/$$i -a \
			! -e $(KERNEL_BUILD_ROOT_DIR)/include/linux/$$i ]; then \
				$(call MESSAGE, Linking  $(KERNEL_BUILD_ROOT_DIR)/include/linux/$$i); \
				ln -sf ../../drivers/char/avm_net_trace/$$i \
					$(KERNEL_BUILD_ROOT_DIR)/include/linux/$$i; \
		fi \
	done
	@for i in $$(find $(KERNEL_BUILD_ROOT_DIR) -name Makefile.26 -printf '%h\n'); do \
		if [ ! -e $$i/Makefile ] ; then \
			$(call MESSAGE, Linking  $$i/Makefile); \
			ln -sf Makefile.26 $$i/Makefile; \
		fi \
	done
	@for i in $$(find $(KERNEL_BUILD_ROOT_DIR) -name Makefile -exec \
		awk '/(obj|subdir)-.*=/{ \
		while (match ($$0,/\\/)) {sub(/\\/," "); getline l;$$0=$$0""l} \
		gsub(/(#.*|.*=)/,""); \
		if (! match ($$0,/,/)) { \
			dirname=substr(FILENAME,1,length(FILENAME)-8); \
			for (i=1;i<=NF;i++) { \
				if (match ($$i,/\.o$$|\$$/)) { \
					$$i=""; \
				} else if (substr($$i,length($$i))!="/") { \
					$$i=$$i"/"; \
				} \
				if ($$i!="") { \
					if (system("test -e "dirname""$$i"Makefile")) { \
						print dirname""$$i"Makefile"; \
					} \
				} \
			} \
		} \
	}' {} '+'|sort -u); do \
		$(call MESSAGE, Creating $$i); \
		mkdir -p $$(dirname "$$i"); \
		[ -h $$i ] && $(RM) $$i; \
		touch $$i; \
	done
	@for i in $$(find $(KERNEL_BUILD_ROOT_DIR) -name Kconfig -exec grep -hs "source.*Kconfig" {} '+'| \
		sed -e 's/\(.*\)#.*/\1/g;s/.*source //g;s/"//g'|sort -u); do \
		if [ ! -e $(KERNEL_BUILD_ROOT_DIR)/$$i ]; then \
			$(call MESSAGE, Creating $(KERNEL_BUILD_ROOT_DIR)/$$i); \
			mkdir -p $(KERNEL_BUILD_ROOT_DIR)/$${i%\/*}; \
			[ -h $(KERNEL_BUILD_ROOT_DIR)/$$i ] && \
				$(RM) $(KERNEL_BUILD_ROOT_DIR)/$$i; \
			touch $(KERNEL_BUILD_ROOT_DIR)/$$i; \
		fi \
	done
	ln -s linux-$(KERNEL_VERSION) $(KERNEL_DIR)/linux
	touch $@

$(KERNEL_DIR)/.configured: $(KERNEL_DIR)/.unpacked $(KERNEL_CONFIG_FILE)
	$(call _ECHO, configuring... )
	cp $(KERNEL_CONFIG_FILE) $(KERNEL_BUILD_ROOT_DIR)/.config
	$(SUBMAKE) $(KERNEL_COMMON_MAKE_OPTIONS) oldconfig
	touch $@

$(KERNEL_DIR)/.prepared: $(KERNEL_DIR)/.configured
	$(SUBMAKE) $(KERNEL_COMMON_MAKE_OPTIONS) prepare
	touch $@

$(KERNEL_HEADERS_DEVEL_DIR)/include/linux/version.h: $(KERNEL_DIR)/.prepared
ifeq ($(strip $(FREETZ_KERNEL_VERSION_2_6_13)),y)
	$(call COPY_KERNEL_HEADERS,$(KERNEL_BUILD_ROOT_DIR),$(KERNEL_HEADERS_DEVEL_DIR),{asm$(_comma)asm-generic$(_comma)linux$(_comma)mtd$(_comma)scsi$(_comma)video})
else
	$(SUBMAKE) $(KERNEL_COMMON_MAKE_OPTIONS) headers_install
	find "$(KERNEL_HEADERS_DEVEL_DIR)" \( -name "..install.cmd" -o -name ".install" \) -exec rm -f \{\} \+
endif
	touch $@

target-toolchain-kernel-headers: $(TARGET_TOOLCHAIN_KERNEL_VERSION_HEADER)
$(TARGET_TOOLCHAIN_KERNEL_VERSION_HEADER): $(TOPDIR)/.config $(KERNEL_HEADERS_DEVEL_DIR)/include/linux/version.h | $(if $(FREETZ_BUILD_TOOLCHAIN),$(TARGET_TOOLCHAIN_STAGING_DIR),$(TARGET_CROSS_COMPILER))
	@$(call COPY_KERNEL_HEADERS,$(KERNEL_HEADERS_DEVEL_DIR),$(TARGET_TOOLCHAIN_STAGING_DIR)/usr)
	@touch $@

$(KERNEL_BUILD_ROOT_DIR)/$(KERNEL_IMAGE): $(KERNEL_DIR)/.prepared $(TOOLS_DIR)/lzma $(TOOLS_DIR)/lzma2eva
	$(call _ECHO, kernel image... )
	$(SUBMAKE) $(KERNEL_COMMON_MAKE_OPTIONS) $(KERNEL_IMAGE)
	touch -c $@

kernel-force:
	$(SUBMAKE) $(KERNEL_COMMON_MAKE_OPTIONS) $(KERNEL_IMAGE)

$(KERNEL_TARGET_DIR)/$(KERNEL_TARGET_BINARY): $(KERNEL_BUILD_ROOT_DIR)/$(KERNEL_IMAGE)
	cp $(KERNEL_BUILD_ROOT_DIR)/$(KERNEL_IMAGE) $(KERNEL_TARGET_DIR)/$(KERNEL_TARGET_BINARY)
	cp $(KERNEL_BUILD_ROOT_DIR)/System.map $(KERNEL_TARGET_DIR)/System-$(KERNEL_LAYOUT)-$(AVM_VERSION).map
	echo "$(KERNEL_SUBVERSION)" > $(KERNEL_TARGET_DIR)/.version-$(KERNEL_LAYOUT)-$(AVM_VERSION)
	touch -c $@

$(KERNEL_DIR)/.modules-$(KERNEL_LAYOUT): $(KERNEL_BUILD_ROOT_DIR)/$(KERNEL_IMAGE)
	@$(call _ECHO, modules... )
	$(SUBMAKE) $(KERNEL_COMMON_MAKE_OPTIONS) modules
	$(SUBMAKE) $(KERNEL_COMMON_MAKE_OPTIONS) modules_install
	touch $@

$(KERNEL_MODULES_DIR)/.modules-$(KERNEL_LAYOUT): $(KERNEL_DIR)/.modules-$(KERNEL_LAYOUT)
	$(RM) -r $(KERNEL_MODULES_DIR)/lib
	mkdir -p $(KERNEL_MODULES_DIR)
	KERNEL_MODULES_SOURCE_DIR=$(KERNEL_DIR)/lib/modules/*/kernel; \
	tar -cf - -C $$KERNEL_MODULES_SOURCE_DIR . | tar -xf - -C $(KERNEL_MODULES_DIR)
	touch $@

kernel-precompiled: pkg-echo-start $(KERNEL_TARGET_DIR)/$(KERNEL_TARGET_BINARY) $(KERNEL_MODULES_DIR)/.modules-$(KERNEL_LAYOUT) pkg-echo-done

kernel-configured: $(KERNEL_DIR)/.prepared

kernel-modules: $(KERNEL_DIR)/.modules-$(KERNEL_LAYOUT)

kernel-help:
	$(SUBMAKE) $(KERNEL_COMMON_MAKE_OPTIONS) help

kernel-menuconfig: $(KERNEL_DIR)/.configured
	$(SUBMAKE) $(KERNEL_COMMON_MAKE_OPTIONS) menuconfig
	-cp -f $(KERNEL_BUILD_ROOT_DIR)/.config $(KERNEL_CONFIG_FILE) && \
	touch $(KERNEL_DIR)/.configured

kernel-xconfig: $(KERNEL_DIR)/.configured
	$(SUBMAKE) $(KERNEL_COMMON_MAKE_OPTIONS) xconfig
	-cp -f $(KERNEL_BUILD_ROOT_DIR)/.config $(KERNEL_CONFIG_FILE) && \
	touch $(KERNEL_DIR)/.configured

kernel-oldconfig: $(KERNEL_DIR)/.configured
	-cp -f $(KERNEL_BUILD_ROOT_DIR)/.config $(KERNEL_CONFIG_FILE) && \
	touch $(KERNEL_DIR)/.configured

kernel-source: $(KERNEL_DIR)/.unpacked

kernel-clean:
	-$(SUBMAKE) $(KERNEL_COMMON_MAKE_OPTIONS) clean

kernel-mrproper:
	-cp -f $(KERNEL_BUILD_ROOT_DIR)/.config $(KERNEL_CONFIG_FILE)
	$(SUBMAKE) $(KERNEL_COMMON_MAKE_OPTIONS) mrproper
	-cp -f  $(KERNEL_CONFIG_FILE) $(KERNEL_BUILD_ROOT_DIR)/.config
		kernel-oldconfig

kernel-dirclean:
	$(RM) -r $(SOURCE_DIR_ROOT)/kernel
	$(RM) $(KERNEL_TARGET_DIR)/.version-*
	$(RM) $(KERNEL_TARGET_DIR)/System*
	$(RM) $(KERNEL_TARGET_DIR)/kernel-*
	$(RM) -r $(KERNEL_TARGET_DIR)/modules-*

kernel-distclean: kernel-dirclean

pkg-echo-start:
	@$(RM) $(ECHO_ITEM_START) $(ECHO_ITEM_BUILD)

pkg-echo-done:
	@$(call _ECHO_DONE)

.PHONY: kernel-configured kernel-modules kernel-menuconfig kernel-oldconfig target-toolchain-kernel-headers
